
#include "uartwrap.h"
#include "mcu.h"
#include "uart_fm3.h"

static void InitPort(void)
{
    /*
     * Set I/O Port
     */

    /* ADC */
    FM3_GPIO->ADE_f.AN31 = 0;  // disable AN31(P22/SOT0_0)

    /* Ch0 */
    FM3_GPIO->PFR2_f.P1 = 1;   // P21/SIN0_0
    FM3_GPIO->EPFR07_f.SIN0S1 = 0;
    FM3_GPIO->EPFR07_f.SIN0S0 = 0;
    FM3_GPIO->PFR2_f.P2 = 1;   // P22/SOT0_0
    FM3_GPIO->EPFR07_f.SOT0B1 = 0;
    FM3_GPIO->EPFR07_f.SOT0B0 = 1;

    /* Ch3 */
    FM3_GPIO->PFR4_f.P8 = 1;   // P48/SIN3_2
    FM3_GPIO->EPFR07_f.SIN3S1 = 1;
    FM3_GPIO->EPFR07_f.SIN3S0 = 1;
    FM3_GPIO->PFR4_f.P9 = 1;   // P49/SOT3_2
    FM3_GPIO->EPFR07_f.SOT3B1 = 1;
    FM3_GPIO->EPFR07_f.SOT3B0 = 1;

    /* Ch4 */
    FM3_GPIO->PFR0_f.P5 = 1;   // P05/SIN4_2
    FM3_GPIO->EPFR08_f.SIN4S1 = 1;
    FM3_GPIO->EPFR08_f.SIN4S0 = 1;
    FM3_GPIO->PFR0_f.P6 = 1;   // P06/SOT4_2
    FM3_GPIO->EPFR08_f.SOT4B1 = 1;
    FM3_GPIO->EPFR08_f.SOT4B0 = 1;
}

/**
 * @brief UARTを初期化する。
 *
 * @retval 0 成功。
 * @retval !0 失敗。
 */
int uartwrap_init(void)
{
    InitPort();
    Uart_Init();
    return 0;
}

/**
 * @brief UARTの読み込みを実行する。
 *
 * @param buf バッファ。
 * @param cnt 読み込みバイト数。
 *
 * @retval 0 成功。
 * @retval !0 失敗。
 */
int uartwrap_read(char *buf, const int cnt, void *extobj)
{
    int32_t n = 0;
    do {
        int32_t r = cnt;
        Uart_BufRx((void *)(buf + n), &r);
        n = n + r;
    } while (n < cnt);
    return n;
}

/**
 * @brief UARTの書き込みを実行する。
 *
 * @param buf バッファ。
 * @param cnt 書き込みバイト数。
 *
 * @retval 0 成功。
 * @retval !0 失敗。
 */
int uartwrap_write(const char *buf, const int cnt, void *extobj)
{
    int32_t n = 0;
    do {
        int32_t w = cnt;
        Uart_BufTx((void *)(buf + n), &w);
        n = n + w;
    } while (n < cnt);
    return n;
}

